/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 12-1 - drawarea.c - Using a drawing widget
 *  
 *  HDR2: Chapter 12, page 370
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 * 1/15/95	Paul Kimball - pkimball@netcom.com
 */

/****** Include required header files *****************************/

#include <X11/Intrinsic.h>       /* The Intrinsics                */
#include <Xm/Xm.h>               /* Resource symbols              */
#include <Xm/Frame.h>            /* Motif XmFrame widget          */
#include <Xm/DrawingA.h>         /* Motif XmDrawingArea widget    */

/****** Forward Declare callback routines *************************/

void HandleResize();             /* Callback - cache new size     */
void HandleExpose();             /* Callback - redraw all         */
void RedrawAll();                /* Function - redraw everything  */
void InitGraphics();             /* Initialize graphics data      */

/****** Declare client_data stuct type(s) *************************/

typedef struct {
  Dimension width, height;       /* Drawing area width and height */
  GC drawing_gc;                 /* Graphics context              */
  Widget canvas;                 /* Drawing area widget ID        */
  XSegment *display_list;        /* List of coordinate pairs      */
  int nlines;                    /* Number of lines in list       */
} GraphicsData;

/****** Start of main() *******************************************/

main(argc,argv)
int  argc;
char **argv;
{
 Widget     toplevel, manager, canvas;          /* Widget tree    */
 XtAppContext  context;
 GraphicsData  gdata;                           /* client_data    */

/*** Init. Toolkit, Appl. Context, Application Shell  *************/

 toplevel = XtAppInitialize( &context, "Drawarea", 
     NULL, 0, &argc, argv, NULL, NULL, 0);

/***  Create widget tree, bind callbacks  ****************************/

 manager = XtCreateManagedWidget( "manager",
     xmFrameWidgetClass, toplevel, NULL, 0);

 gdata.canvas = XtCreateManagedWidget( "canvas",
     xmDrawingAreaWidgetClass, manager, NULL, 0);
 XtAddCallback( gdata.canvas, XmNresizeCallback,
     HandleResize, (XtPointer)&gdata );
 XtAddCallback( gdata.canvas, XmNexposeCallback,
     HandleExpose, (XtPointer)&gdata );

 XtRealizeWidget(toplevel);        /* Force widgets to get windows   */
 InitGraphics(&gdata);             /* Initialize graphics resources  */
 XtAppMainLoop(context);
}

/****** Callback routines and utilities ******************************/

void InitGraphics(gdp)
GraphicsData* gdp;
{
  Pixel color;
  XGCValues gcvalues;
  static XSegment stardata[]={          /* Data defining a star      */
  { 17, 85, 87, 31 },                   /* Coords are percentage of  */
  { 87, 31, 13, 31 },                   /* width, height             */
  { 13, 31, 83, 85 },
  { 83, 85, 50, 5 },
  { 50, 5, 17, 85 } };
  gdp->display_list = stardata;
  gdp->nlines = XtNumber(stardata);
  XtVaGetValues( gdp->canvas,           /* Store initial geometry    */
  XmNwidth, &(gdp->width),              /* Needed for initial expose */
  XmNheight, &(gdp->height),
  XmNforeground, &color,                /* Get drawing pixel value   */
  NULL);  
  gcvalues.foreground = color;
  gdp->drawing_gc = XCreateGC(XtDisplay(gdp->canvas),
  XtWindow(gdp->canvas), GCForeground, &gcvalues );
}

/*********************************************************************/

void HandleResize(w, client_data, call_data)
Widget   w;
XtPointer client_data;
XtPointer call_data;
{
  GraphicsData* gdp = (GraphicsData*)client_data;
  XtVaGetValues( w, 
  XmNwidth, &(gdp->width),              /* Get new size from canvas  */
  XmNheight, &(gdp->height), NULL);
  /* Redraw at new scale */
  if (XtIsRealized(gdp->canvas)) RedrawAll( gdp );
}

/*********************************************************************/

void HandleExpose(w, client_data, call_data)
Widget   w;
XtPointer client_data;
XtPointer call_data;
{
  GraphicsData* gdp = (GraphicsData*)client_data;
  RedrawAll( gdp );
}

/*********************************************************************/

void RedrawAll(gdp)
GraphicsData* gdp;
{
  int i;
  Widget canvas = gdp->canvas;

  /* Clear window since it won't clear if widget gets smaller */
  XClearWindow(XtDisplay(canvas), XtWindow(canvas) );

  /* Redraw everything */
  XDrawLine( XtDisplay(canvas), XtWindow(canvas), gdp->drawing_gc,
      10, 10, 32768, 32768);

  for(i=0; i<gdp->nlines; i++) {
    XDrawLine( XtDisplay(canvas), XtWindow(canvas), gdp->drawing_gc,
        ( gdp->width * gdp->display_list[i].x1 )/100,
        ( gdp->height * gdp->display_list[i].y1 )/100,
        ( gdp->width * gdp->display_list[i].x2 )/100,
        ( gdp->height * gdp->display_list[i].y2 )/100 );
  }
}
