/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Bonus Example - pgrab.c - Demonstrates grabbing a function key
 *  
 *  HDR2: 
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

/*** Include Required Header Files  *********************************/

#include <X11/Intrinsic.h>          /* The Intrinsics               */
#include <X11/keysym.h>             /* X keysyms                    */
#include <Xm/Xm.h>                  /* Motif resource symbols       */
#include <Xm/BulletinB.h>           /* Motif XmBulletinBoard widget */
#include <Xm/Label.h>               /* Motif XmLabel widget         */
#include <Xm/Text.h>                /* Motif XmText widget          */
#include <Xm/PushB.h>               /* Motif XmPushButton widget    */

/***** Declare Global Variables  ************************************/

XtAppContext context;               /* Application context          */
Widget       toplevel, manager,     /* Widget tree                  */
             top_label, text_area,
             first_button, second_button;

Arg       arglist[10];              /* An ArgList of ten Args       */
int       i;                        /* An array index               */

/***** Set up Translations ******************************************/

void ExitAction();

static XtActionsRec actions[] = {
    { "ExitAction", ExitAction }
};

static char translations[]= 
    "#override\n\
    Ctrl<Key>q: ExitAction()";

/***** Main Program  ***********************/

main(argc,argv)
int  argc;
char **argv;
{

KeyCode  *keycodes;
Cardinal num_keycodes;
XtTranslations xlations;

/******************************************************************
 *  Init. Toolkit, Application Context and Application Shell      */

/* Set application title     */

 i = 0;
 XtSetArg(arglist[i], XmNtitle, "Control Panel"); i++; 
 toplevel = XtAppInitialize (    /* Create application shell  */
     &context, "Panel",      /* Application name, class   */
     NULL, 0,           /* options, num_options      */
     &argc, argv,        /* Command line              */
     NULL,            /* Fallback resources        */
     arglist, i );        /* Initial resources         */

/*****  Create widget tree, bind callbacks *****/

 /*  Here is the easy way to reserve Ctrl-Q as a function key for the use of
  *  the manager widget.  Registering 'ExitAction()' as a grab action
  *  will cause the toolkit to establish a passive grab on any physical
  *  keys that could invoke the action.  The toolkit automatically
  *  checks the translation table to see which keys are required
  */
 XtRegisterGrabAction(ExitAction, KeyPressMask, True, GrabModeAsync,
    GrabModeAsync);
    
    
 XtAppAddActions(context, actions, XtNumber(actions));
 xlations = XtParseTranslationTable(translations);
 XtOverrideTranslations(toplevel, xlations);

 i = 0;                                     /* Init ArgList     */
 manager = XtCreateManagedWidget(           /* XmBulletinBoard */
     "manager", xmBulletinBoardWidgetClass,
     toplevel, arglist, i);

#if 0

 /*  Here is another way to reserve Ctrl-Q as a function key for the use of 
  *  the manager widget - you can try this instead by replacing the #if
  *  just above with '#if 1'
  */
  
 /* First, determine which physical keys could mp to the KeySym for "Q" */
 XtKeysymToKeycodeList(XtDisplay(toplevel), XK_Q, &keycodes,
    &num_keycodes);

 /* Then, grab them all */
 for (i=0; i<num_keycodes; i++){
 XtGrabKey(manager, keycodes[i], ControlMask, True, GrabModeAsync,
    GrabModeAsync);
 }
 /* Now, Ctrl-Q pressed with focus anywhere below the manager widget
    will be processed by manager. */
    
#endif 


 i = 0;                                     /* Init ArgList     */
 XtSetArg(arglist[i], XmNx, 10); i++;       /* Set X position   */
 XtSetArg(arglist[i], XmNy, 10); i++;       /* Set Y position   */
 top_label = XtCreateManagedWidget(         /* XmLabel          */
     "topLabel", xmLabelWidgetClass,
     manager, arglist, i);

 i = 0;                                     /* Init ArgList     */
 XtSetArg(arglist[i], XmNx, 10); i++;       /* Set X position   */
 XtSetArg(arglist[i], XmNy, 40); i++;       /* Set Y position   */
 XtSetArg(arglist[i], XmNvalue,             /* Initial value of */
     "Type text here"); i++;                /* editable text    */
 text_area = XtCreateManagedWidget(         /* XmText           */
     "textArea", xmTextWidgetClass,
     manager, arglist, i);

 i = 0;                                     /* Init ArgList     */
 XtSetArg(arglist[i], XmNx, 10); i++;       /* Set X position   */
 XtSetArg(arglist[i], XmNy, 90); i++;       /* Set Y position   */
 XtSetArg(arglist[i], XmNwidth, 100); i++;  /* Set width        */
 first_button = XtCreateManagedWidget(      /* XmPushButton     */
     "firstButton", xmPushButtonWidgetClass,
     manager, arglist, i);

 i = 0;                                     /* Init ArgList     */
 XtSetArg(arglist[i], XmNx, 120); i++;      /* Set X position   */
 XtSetArg(arglist[i], XmNy, 90); i++;       /* Set Y position   */
 XtSetArg(arglist[i], XmNwidth, 100); i++;  /* Set width        */
 second_button = XtCreateManagedWidget(     /* XmPushButton     */
     "secondButton", xmPushButtonWidgetClass,
     manager, arglist, i);

/***** Manage, Realize initial Widget Tree ***/

 XtRealizeWidget(toplevel);

/***** Start Event Parsing Loop **************/

 XtAppMainLoop(context);
}


/***** Callback routines and Action Procs  *********************/


void ExitAction(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    exit();
}
