/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 10-3 - select.c - Toolkit selection
 *  
 *  HDR2: Chapter 10, page 303
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

/*****  Include Required Header Files    *****************************/

#include <X11/Intrinsic.h>        /* The Intrinsics                  */
#include <X11/Xatom.h>            /* Intrinsics pre-defined atoms    */
#include <Xm/Xm.h>                /* Motif resource symbols          */
#include <Xm/RowColumn.h>         /* Motif XmRowColumn widget        */
#include <Xm/PushB.h>             /* Motif XmPushButton widget       */
#include <Xm/ToggleB.h>           /* Motif XmToggleButton widget     */

/*****  Declare Global Variables   ***********************************/

XtAppContext  context;            /* Application context             */
Widget toplevel, manager;         /* Widget tree                     */
Widget button[3], exit_btn;
String sel_string;                /* Selection converted to String   */
int    sel_button;                /* Index of the selected button    */
Atom   TEXT, TARGETS, TIMESTAMP;  /* Selection targets               */

/*****  Forward declare callback routines   **************************/

void ExitRoutine();           /* Callback - exit application         */
void OwnSelection();          /* Callback - take selection ownership */
Boolean ConvertSelection();   /* Callback - supply selection value   */
void LoseSelection();         /* Callback - reset if selection lost  */
void RequestSelection();      /* Ev. handler - ask for selection     */
void ReceiveSelection();      /* Callback - receive selection value  */

/*****  Start of main()   ********************************************/

main(argc,argv)
int argc;
char **argv;
{

/*****  Init. Toolkit, Appl. Context, Application Shell  *************/

 toplevel = XtAppInitialize(&context, "Select",
     NULL, 0, &argc, argv, NULL, NULL, 0);

/*****  Create widget tree, bind callbacks  **************************/

 manager = XtCreateManagedWidget("manager",           /* XmRowColumn */
     xmRowColumnWidgetClass, toplevel, NULL, 0);      /*  manager    */

 button[0] = XtCreateManagedWidget("button0",         /* Create 3    */
     xmToggleButtonWidgetClass, manager, NULL, 0);    /*  toggles    */

 button[1] = XtCreateManagedWidget("button1",
     xmToggleButtonWidgetClass, manager, NULL, 0);

 button[2] = XtCreateManagedWidget("button2",
     xmToggleButtonWidgetClass, manager, NULL, 0);

 XtAddCallback( button[0], XmNvalueChangedCallback,   /* Any button  */
     OwnSelection, (XtPointer)0 );                    /*  attempts   */
 XtAddCallback( button[1], XmNvalueChangedCallback,   /*  to own     */
     OwnSelection, (XtPointer)1 );                    /*  selection  */
 XtAddCallback( button[2], XmNvalueChangedCallback,   /*  when       */
     OwnSelection, (XtPointer)2 );                    /*  activated  */

 XtAddEventHandler( button[0], ButtonPressMask,       /* Any button  */
     False, RequestSelection, NULL);                  /*  asks for   */
 XtAddEventHandler( button[1], ButtonPressMask,       /*  selection  */
     False, RequestSelection, NULL);                  /*  if MB2 is  */
 XtAddEventHandler( button[2], ButtonPressMask,       /*  pressed    */
     False, RequestSelection, NULL);                  /*  inside     */

 exit_btn = XtCreateManagedWidget("exitBtn",          /* Exit button */
     xmPushButtonWidgetClass, manager, NULL, 0);

 XtAddCallback( exit_btn, XmNactivateCallback,        /* Call back   */
     ExitRoutine, NULL );                             /*  to exit    */

 /***  Intern any Atoms that aren't pre-defined               */
 TARGETS = XInternAtom(XtDisplay(toplevel), "TARGETS", False);
 TEXT = XInternAtom(XtDisplay(toplevel), "TEXT", False);
 TIMESTAMP = XInternAtom(XtDisplay(toplevel), "TIMESTAMP", False);

 XtRealizeWidget(toplevel);                  /* Realize all          */
 XtAppMainLoop(context);                     /* Start main loop      */
}

/*** Callback Routines ***********************************************/

void                                        /* Take sel. ownership   */ 
OwnSelection(w, client_data, call_data)
Widget w;                                   /* Widget ID of toggle   */
XtPointer client_data;                      /* Index in toggle array */
XtPointer call_data;                        /* Unused                */
{
 Boolean state, gotit;
 Time   sel_time = XtLastTimestampProcessed(XtDisplay(w));

 XtVaGetValues(w, XmNset, &state, NULL);    /* Get new toggle state  */

 if(state){                                 /* Button was toggled on */
   sel_button = (int)client_data;           /* Remember index        */
   gotit = XtOwnSelection(                  /* Take ownership        */
      w,                                    /* Selection owner       */
      XA_PRIMARY,                           /* Own PRIMARY Selection */
      sel_time,                             /* Timestamp from event  */
      ConvertSelection,                     /* Call to supply value  */
      LoseSelection,                        /* Call if ownership lost*/
      (XtSelectionDoneProc)NULL );          /* Call when xfer is done*/ 

 /* Failed, so remove highlight  */

   if (!gotit) XtVaSetValues(w, XmNset, False, NULL); 
 }
 /* Button toggled off by user, so disown selection */

 else XtDisownSelection(w, XA_PRIMARY, sel_time);
}

/*********************************************************************/

void                                    /* Reset toggles             */
LoseSelection(w, selection)             /*  if ownership is lost     */
Widget w;                               /* Widget losing selection   */
Atom   *selection;                      /* Identifies lost selection */
{
 /* Remove highlighting to indicate selection no longer active       */

 XtVaSetValues(w, XmNset, False, NULL);

 /* Free any storage pre-allocated to transfer (none in this case)   */ 
}

/*********************************************************************/

Boolean                                 /* Convert selection and     */
ConvertSelection(w, selection, target,  /*  return True if success   */
 type_return, value, length, format)
Widget w;                               /* Widget that owns selection*/
Atom *selection;                        /* Requested selection       */
Atom *target;                           /* Requested target          */
Atom *type_return;                      /* Return data type          */
XtPointer *value;                       /* Return converted value    */
unsigned long *length;                  /* Return length             */
int *format;                            /* Return format             */
{
 Atom *tarray;                          /* Array of possible targets */
 int MAX_TARGETS=8, tcount = 0;
 XmString xms;                          /* XmString                  */
 Boolean status;                        /* Status variable           */

 /* check which selection this is, if it makes a difference          */

 if (*selection != XA_PRIMARY) return;

 /* Now check target Atom, respond to mandatory targets              */

 if( *target == TARGETS ){    

   /* requester asked which targets we support so assemble array     */

   tarray = (Atom*)XtMalloc((unsigned)(MAX_TARGETS*sizeof(Atom)));
   tarray[tcount++] = TARGETS;
   tarray[tcount++] = TIMESTAMP;
   tarray[tcount++] = TEXT;
   tarray[tcount++] = XA_STRING;
   tarray[tcount++] = XA_INTEGER;
   *value = (XtPointer)tarray;          /* Value is array of targets */
   *type_return = XA_ATOM;              /* Type is Atom list         */
   *format = 32;                        /* 32-bit data               */
   *length = (unsigned long)tcount;     /* Length is in 32-bit units */
   return True;                         /* Successfully converted    */
 }
 else if ( *target == XA_INTEGER){

   /* requester wants index of selected button; copy to temp storage */

   int *temp = (int*)XtMalloc((unsigned) sizeof(int));
   *temp = sel_button;
   *value = (XtPointer)temp;            /* Index of selected button  */
   *type_return = XA_INTEGER;           /* Index is an integer       */
   *format = 32;                        /* 32-bit data               */
   *length = 1;                         /* Length is in 32-bit units */
   return True;                         /* Successfully converted    */
 }
 else if ( (*target == XA_STRING) | (*target == TEXT) ){      

   /* requester wants to know value of label string so      */
   /* Retrieve XmString from button and convert to string   */

   XtVaGetValues(button[sel_button], XmNlabelString, &xms, NULL);
   status = XmStringGetLtoR(xms, XmFONTLIST_DEFAULT_TAG, &sel_string);  
   XmStringFree(xms);

   if ( status ) { 
     *value = (XtPointer)sel_string;    /* Value is char string      */
     *type_return = XA_STRING;          /* Returned type is String   */
     *format = 8;                       /* 8-bit character data      */
     *length = strlen(sel_string);      /* Length is in 8-bit units  */
     return True;                       /* Successfully converted    */
   }
   else return False;                   /* Conversion failed         */
 }
 else  return False;                    /* Unrecognized target: fail */
}

/*********************************************************************/

void                                    /* Ask for current selection */
RequestSelection(w, client_data, event, dispatch)
Widget w;
XtPointer client_data;
XEvent *event;
Boolean *dispatch;
{
 if ((event->type != ButtonPress )
    ||( event->xbutton.button != 2 )) return;
 XtGetSelectionValue(w,                 /* Ask for selection value   */
 XA_PRIMARY,                            /* Identifies the selection  */
 XA_STRING,                             /* Desired target            */
 ReceiveSelection,                      /* Callback to receive value */
 NULL,                                  /* Client data (unused )     */
 XtLastTimestampProcessed(XtDisplay(w))); /* Time of request    */
}
void                                    /* Receive selection value   */
ReceiveSelection(w, client_data, selection, type, 
 value, length, format)
Widget w;                               /* Receiving widget          */
XtPointer client_data;                  /* Client data               */
Atom *selection;                        /* Identifies selection      */
Atom *type;                             /* Returned data type        */
XtPointer value;                        /* Returned selection value  */
unsigned long *length;                  /* Returned selection length */
int *format;                            /* Returned selection format */
{
 char* strbuf;
 XmString xms;

 if (*type == XT_CONVERT_FAIL) return;  /* Selection was unavailable */
 else if (*type == XA_STRING){          /* Selection retrieved OK    */

   /* Copy value to new storage and NULL-terminate */   

   strbuf = (char*)XtMalloc((unsigned)(*length + 1)); 
   bcopy(value, strbuf, *length);
   strbuf[*length] = (char)NULL;
   XtFree(value);                       /* Free memory               */

   /* Convert to XmString and put in button label */

   xms = XmStringCreateLocalized(strbuf);
   XtVaSetValues(w, XmNlabelString, xms, NULL);
   XmStringFree(xms);
   XtFree(strbuf);                      /* Free memory               */
 }
}

/*********************************************************************/

void                                    /* Called to exit            */
ExitRoutine(w, client_data, call_data)
Widget w;                               /* Unused                    */
XtPointer client_data, call_data;       /* Unused                    */
{
 exit(1);                               /* Exit                      */
}

